/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl>
 *
 */


#include <xacml.h>
#include <lcmaps/lcmaps_types.h>


typedef struct subject_attr_s {
    char *subject_x509_id;		/* needs free */
    char *subject_x509_issuer;		/* needs free */
    char *ca_issuer_dn;			/* needs free */
    char validity_not_before[21];
    char validity_not_after[21];
    char *certificate_serial_number;	/* needs free */
    char *ca_serial_number;		/* needs free */
    int numpols;
    char **ca_policy_oid;		/* needs free */
    char *cert_chain;
    /*char *subject_condor_canonical_name_id*/
} subject_attr_t;


/* B: prototypes */



/* Helper funcs */

int pep_register_obligation_handlers (xacml_request_t);
int pep_construct_request_subject (xacml_request_t   request,
			    subject_attr_t    *d,
			    lcmaps_vomsdata_t *vomsdata,
			    int               cert_client_side_verified,
			    int               voms_client_side_verified
                          );
int pep_construct_request_resource (xacml_request_t   request,
                                    const char *      resource_node_type,
                                    const char *      host_dns_name,
                                    const char *      resource_id,
                                    const char *      resource_x509_id,
                                    const char *      resource_x509_issuer
                                   );
int pep_construct_request_action   (xacml_request_t   request,
                                    const char *      action_type
                                   );

#if 0
int get_gidlist(
        const char * username,
        int * ngroups,
        gid_t ** group_list
    );
#endif

/* Setter for use_authorization_only */
void pep_set_use_authorization_only(int value);

/* Getter for use_authorization_only */
int pep_get_use_authorization_only(void);


/* Obligation handlers */

int
default_handler(
    void *                              handler_arg,
    const xacml_response_t              response,
    const char *                        obligation_id,
    xacml_effect_t                      fulfill_on,
    const char *                        attribute_ids[],
    const char *                        datatypes[],
    const char *                        values[]);


/* example */
int
local_user_name_handler(
    void *                              handler_arg,
    const xacml_response_t              response,
    const char *                        obligation_id,
    xacml_effect_t                      fulfill_on,
    const char *                        attribute_ids[],
    const char *                        datatypes[],
    const char *                        values[]);
/* example */

int
Username_handler (
    void *                              handler_arg,
    const xacml_response_t              response,
    const char *                        obligation_id,
    xacml_effect_t                      fulfill_on,
    const char *                        attribute_ids[],
    const char *                        datatypes[],
    const char *                        values[]);

int
Account_handler (
    void *                              handler_arg,
    const xacml_response_t              response,
    const char *                        obligation_id,
    xacml_effect_t                      fulfill_on,
    const char *                        attribute_ids[],
    const char *                        datatypes[],
    const char *                        values[]);

int
UIDGID_handler (
    void *                              handler_arg,
    const xacml_response_t              response,
    const char *                        obligation_id,
    xacml_effect_t                      fulfill_on,
    const char *                        attribute_ids[],
    const char *                        datatypes[],
    const char *                        values[]);

int
SecondaryGIDs_handler (
    void *                              handler_arg,
    const xacml_response_t              response,
    const char *                        obligation_id,
    xacml_effect_t                      fulfill_on,
    const char *                        attribute_ids[],
    const char *                        datatypes[],
    const char *                        values[]);

int
AFSToken_handler (
    void *                              handler_arg,
    const xacml_response_t              response,
    const char *                        obligation_id,
    xacml_effect_t                      fulfill_on,
    const char *                        attribute_ids[],
    const char *                        datatypes[],
    const char *                        values[]);
/* Obligation handlers */


int getOHUIDGIDFired(void);
int getOHSecondaryGIDsFired(void);
int getOHUsernameFired(void);
int getOHAccountFired(void);

int get_uid_list(uid_t *uids[], size_t *count);
int get_pgid_list(gid_t *pgids[], size_t *count);
int get_sgid_list(gid_t *sgids[], size_t *count);

void clean_uidgid_lists(void);

const char *decision_str(xacml_decision_t decision);
const char *fulfill_on_str(xacml_effect_t fulfill_on);


/* E: prototypes */

